'use strict';

var axios = require('axios');

var config = {
  isBrowser: true,
  prod: {
    basicBaasUrl: '//baas.ele.me/serverless'
  },
  alpha: {
    basicBaasUrl: '//httpizza.alpha.elenet.me/fe.baas_api/serverless'
  },
  beta: {
    basicBaasUrl: '//httpizza.beta.elenet.me/fe.baas_api/serverless'
  }
};

if (typeof module !== 'undefined' && module.exports) {
  config.isBrowser = false;
}

var assert = function(bool, message) {
  if (!bool) {
    throw new Error(message);
  }
  return;
};

var Server = function(opts) {
  var appid = opts.appid;
  var serviceName = opts.serviceName;

  assert(typeof opts === 'object', 'opts type is not object');
  assert(typeof appid === 'string', 'appid type is not object');
  assert(typeof serviceName === 'string', 'serviceName type is not object');
  this.env = opts.env || 'alpha';

  assert(typeof this.env === 'string', 'env type is not string');
  assert(['alpha', 'beta', 'production'].indexOf(this.env) >= 0, 'env' + this.env + 'is invalid');

  this.serviceName = [appid, this.env, serviceName].join('-');

  this.BaasEnv = opts.BaasEnv || 'prod';

  assert(typeof this.BaasEnv === 'string', 'BaasEnv type is not string');
  assert(['alpha', 'beta', 'prod'].indexOf(this.BaasEnv) >= 0, 'BaasEnv' + this.BaasEnv + 'is invalid');

  var protocol = opts.protocol;

  if (protocol) {
    assert(typeof protocol === 'string', 'protocol must be string');
    assert(['http', 'https'].indexOf(protocol) >= 0, `error protocol string ${protocol}`);
    this.protocol = protocol + ':';
  } else {
    if (config.isBrowser) {
      this.protocol = '';
    } else {
      this.protocol = 'https:';
    }
  }
};

Server.prototype.run = function(funcName, data, options) {
  data = data || {};
  options = options || {};
  assert(typeof funcName === 'string', 'funcName is not string');
  assert(typeof data === 'object', `${funcName} data type ${typeof data} is not object`);

  var queryParameters = data.queryParameters || {};
  var pathParameters = data.pathParameters || {};
  var method = data.method || 'GET';

  assert(typeof queryParameters === 'object', 'data.queryParameters is not object');
  assert(typeof pathParameters === 'object', 'data.pathParameters is not object');
  assert(typeof method === 'string', 'data.method is not string');

  var basicBaasUrl = config[this.BaasEnv]['basicBaasUrl'];

  var url = this.protocol + basicBaasUrl + '/' + this.serviceName + '/' + funcName;

  queryParameters = JSON.stringify(data.queryParameters || {});
  pathParameters = JSON.stringify(data.pathParameters || {});

  var body = data.body || {};
  var requestOption = {
    method: method,
    url: url,
    params: {
      queryParameters: queryParameters,
      pathParameters: pathParameters
    },
    data: {
      body: body
    }
  };
  return axios(Object.assign({}, requestOption, options))
  .then(function(res) { return res.data; })
  .catch(function(res) { throw new Error(JSON.stringify(res.response.data));});
};

module.exports = Server;
